
;Copyright (c) 2006 David Marion, All rights reserved.
;
;Redistribution and use in source and binary forms, with or without modification, are permitted 
;provided that the following conditions are met:
;
;    * Redistributions of source code must retain the above copyright notice, this list of 
;      conditions and the following disclaimer.
;    * Redistributions in binary form must reproduce the above copyright notice, this list of 
;      conditions and the following disclaimer in the documentation and/or other materials 
;      provided with the distribution.
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
;IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
;FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
;CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
;CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
;SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
;OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
;POSSIBILITY OF SUCH DAMAGE.



EXTERN cmain
EXTERN printf
EXTERN halt_message
GLOBAL _start
GLOBAL _halt



STACK_SIZE equ 0x1000
MULTIBOOT_MAGIC equ 0x1BADB002
MULTIBOOT_FLAGS equ 0x00000003
MULTIBOOT_CHECKSUM equ -(MULTIBOOT_MAGIC + MULTIBOOT_FLAGS)



SECTION .text

_start:
  jmp  multiboot_entry

; The GRUB Multiboot header for ELF
align 4
multiboot_header:
  dd MULTIBOOT_MAGIC
  dd MULTIBOOT_FLAGS
  dd MULTIBOOT_CHECKSUM

multiboot_entry:
  ; Initialize the stack pointer.
  mov ecx,(stack+STACK_SIZE)
  mov esp,ecx

  ; Reset EFLAGS. 
  mov ecx,0
  push ecx
  popf

  push ebx ; Pointer to the Multiboot information structure.
  push eax ; Multiboot magic (validate structure pointer).
  call cmain  ; I do not expect this to return.


_halt:
  mov eax,[halt_message]
  push eax
  call printf

spin:
  hlt
  jmp spin



; Nothing else should be in this data section
SECTION .data
stack:
  times STACK_SIZE db 0

