// irq.h
// Defines IRQ Methods
// Part of Plato OS
// Code taken from OSDev.org

// Typedefs
typedef unsigned long dword;
typedef unsigned short word;
typedef unsigned char byte;

/*-------------------------------*/
/* Memory Access                 */
/*-------------------------------*/

// FAR_PEEKx
static __inline__ dword farpeekl(word sel,void *off)
{
  dword ret;
  asm("push %%fs;mov %1,%%fs;"
      "mov %%fs:(%2),%0;"
      "pop %%fs":"=r"(ret):"g"(sel),"r"(off));
  return ret;
}

// FAR_POKEx
static __inline__ void farpokeb(word sel, void *off, byte v)
{
  asm("push %%fs; mov %0,%%fs;"
      "movb %2,%%fs:(%1);"
      "pop %%fs": :"g"(sel),"r"(off),"r"(v));
}

/*-------------------------------*/
/* I/O Access                    */
/*-------------------------------*/

// OUTx
static __inline__ void outb(unsigned short port, unsigned char val)
{
   asm volatile("outb %0,%1"::"a"(val), "Nd" (port));
}

// INx
static __inline__ unsigned char inb(unsigned short port)
{
   unsigned char ret;
   asm volatile ("inb %1,%0":"=a"(ret):"Nd"(port));
   return ret;
}

// IO_WAIT
static __inline__ void io_wait(void)
{
   asm volatile("jmp 1f;1:jmp 1f;1:");
}

static __inline__ void io_wait_2(void)
{
   asm volatile("outb %%al, $0x80" : : "a"(0));
   // port 0x80 is used for 'checkpoints' during POST.
   // linux kernel seems to think it's free for use :-/
}

/*-------------------------------*/
/*  Interrupt-Related Functions  */
/*-------------------------------*/

// Enabled?
static __inline__
int irqEnabled()
{
        int f;
        asm volatile ("pushf;popl %0":"=g" (f));
        return f & (1<<9);
}

// Acknowledge
static __inline__
void irqUnlock(int no)
{
  /*             Val, Port */
  if (no>7) outb(0x20,0xa0);
  outb(0x20,0x20);
}

// LIDT - Define a new interrupt table
void lidt(void *base, unsigned int size) {
   unsigned int i[2];

   i[0] = size << 16;
   i[1] = (unsigned int) base;
   asm ("lidt (%0)": :"p" (((char *) i)+2));
}

/*-------------------------------*/
/*  CPU-Related Functions        */
/*-------------------------------*/

// CPUID
/** issue a single request to CPUID. Fits 'intel features', for instance
 */
static inline void cpuid(int code, dword *a, dword *d) {
  asm volatile("cpuid":"=a"(*a),"=d"(*d):"0"(code));
}

// RDTSC
static inline void rdtsc(dword *upper, dword *lower)
{
  asm volatile("rdtsc\n" : "=a"(*lower), "=d"(*upper));
}

// READ_CRx
static inline unsigned read_cr0() {
   unsigned val;
   asm volatile("mov %%cr0, %0":"=r"(val));
   return val;
}

/* PGFLUSHTLB
static __inline__
void pgFlushOneTlb(void *m)
{
  asm volatile("invlpg %c"::"m" (*m));
}
*/
