/*
Plato OS - Development. Experimental. Not reccomended for home use.
Copyright (C) 2006  Plato Project

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

I like to give credit where credit is due, so thanks to the following people:
	d06 of SF.net for lots of help.
	OSDev.net for some code.
	SaturnOS for some code.
*/

#include "common.h"
#include "shell.h"         // Simple Shell
#include "prototypes.h"       // Function Prototypes
#include "remap_pic.h"       // To remap the PIC

#define newline (char *)0xA     // So we don't have to type (char *)0xA every time we want a newline
#define bell (char *)0x7      // So we don't have to type (char *)0x7 every time we want a beep

int main()            // Main
{
	remap_pics(0x20, 0x28);      // Remap the PIC
	boot();             // Startup tests, etc.
	shell();        // Simple Shell
}

void boot()            // This is the main boot sequence
{
	int boottime[3];
        boottime[1]=time(NULL);

        printf("%c%cTesting Processor Integrity. Stand By. %c",newline,newline,newline);
        procint();
        // Here's where we'll load drivers and such.
        printf("%c%c%cPlato Operating System. Based on streamlined PASCAL and C++ %c",newline,newline,newline,newline);
        printf("(C)2006 Plato Project%chttp://plato-os.sourceforge.net %c %c",newline,newline,newline);

        printf("System wake at %i %c",boottime[1],newline);
        printf("Loading Plato OS...%c",newline);
        printf("%cTesting Bell...%c",bell,newline);
        boottime[2]=time(NULL);
        boottime[3]=boottime[2]-boottime[1];
        printf("Plato OS Startup completed at %i, Time: %i seconds. %c %c",boottime[2],boottime[3],newline,newline);
        printf("System has been deemed stable. Proceed. %c%c",newline,newline);
	help();
        wait(100000);
        // Either the user can enter a command to start the GUI, or we can load it here automatically.
}

void longabout()           // Tell the user about Plato OS
{
	printf("%c%cPlato OS - System Operation Software - Based on streamlined PASCAL and C++ %c",newline,newline,newline);
        printf("(C)2006 Plato Project\nhttp://plato-os.sourceforge.net %c %c",newline,newline);
        printf("Protected by the GNU GPL. On the disk is a copy of this liscense, gnu.txt %c %c",newline,newline);
        printf("Authors, Development and Acknowledgement%c===================================== %c",newline,newline);
        printf("SUPER THANKS to the SaturnOS Project for the awesome source code ideas!!! %c",newline);
        printf("Lead Developer - Dustin Weitkunat %c%c",newline,newline);
}

void halt()                   // Stop the system
{
	printf("%c Complete system halt in progress. Stand By. %c",bell,newline);
        // This is where all the drives will be unmounted, 
        // the GUI will shut off, and any hardware drivers will stop.
        wait(100000);
        printf("%c System halted. Reset your computer manually. %c",newline,newline);
}

void procint()                    // Test whether the computer is powerful enough for Plato
{
	for(int count = 0; count < 15000; count++)
        {
		printf("Cyclic Processor Test In Progress. CYCLE: %d                               %c",count,newline);
	        wait(100);
        }
        clear();
}

void wait(long int length)                // Waste processor time, creates a wait
{
	for(int count = 0; count < length; count++);
}

void clear()                          // Fill screen with blanks
{
	for(int count = 0; count < 2001; count++)
	{
		printf(" ");
	}
}

void update_cursor(int row, int col)       // Move cursor
{
	unsigned short position = (row*80) + col;

	outb(0x3d4, 0x0f);
	outb(0x3d5, (unsigned char)(position&0xff));
	outb(0x3d4, 0x0e);
	outb(0x3d5, (unsigned char)((position>>8)&0xff));
}

void help()                // Help on Plato
{
	printf("%c%cPlato OS %c",newline,newline,newline);
        printf("h - Halt the system (NOTE: For now the system just hangs) %c",newline);
        printf("a - List information and credits %c",newline);
	printf("t - Get system time. %c",newline);
	printf("i - Display this help. %c%c",newline,newline);
}

void redrawMain()             // Redraw the main screen
{
	printf("%c%c%cPlato Operating System. Based on streamlined PASCAL and C++ %c",newline,newline,newline,newline);
        printf("(C)2006 Plato Project%chttp://plato-os.sourceforge.net %c %c",newline,newline,newline);
	help();
        wait(100000);
}
